import hmac
import hashlib
import requests
from requests.auth import OAuth1
from oauthlib.oauth1.rfc5849 import SIGNATURE_TYPE_BODY

api_version = '1.0'

class Tambur(object):
    def __init__(self, api_key=None, app_id=None, secret=None, api_host='api.tambur.io'):
        self.api_key = str(api_key)
        self.app_id = str(app_id)
        self.secret = str(secret)
        self.api_host = str(api_host)
        self.oauth = OAuth1(unicode(self.api_key), client_secret=unicode(self.secret), signature_type=SIGNATURE_TYPE_BODY)
        self.url = 'http://'+self.api_host+'/app/'+self.app_id

    def publish(self, stream, message):
        params = {
            'api_version' : api_version,
            'message' : message
        }

        r = requests.post(self.url + '/stream/'+str(stream), data=params, auth=self.oauth)
        r.raise_for_status()
        return True

    def generate_auth_token(self, stream, subscriber_id):
        return self.__generate_mode_token('auth', stream, subscriber_id)

    def generate_presence_token(self, stream, user_id, subscriber_id):
        return self.__generate_mode_token('presence', stream + ':' + user_id, subscriber_id)

    def generate_direct_token(self, stream, user_id, subscriber_id):
        return self.__generate_mode_token('direct', stream + ':' + user_id, subscriber_id)

    def __generate_mode_token(self, mode, props, subscriber_id):
        mode_string = self.api_key + ':' + self.app_id + ':' + mode + ':' + props + ':' + subscriber_id
        return hmac.new(self.secret, mode_string, hashlib.sha1).hexdigest()



import unittest
import requests
import random
import json
from tambur import Tambur

class TamburPublishTest(unittest.TestCase):
    def setUp(self):
        self.s = requests.session()
        r = self.s.get('http://wsbot.tambur.io/credentials')
        credentials = json.loads(r.text)
        api_key = credentials['api_key']
        app_id = credentials['app_id'].encode('ascii')
        secret = credentials['secret'].encode('ascii')
        r = self.s.get('http://wsbot.tambur.io/subscriber_id')
        self.subscriber_id = r.text
        self.tambur = Tambur(api_key=api_key, app_id=app_id, secret=secret)

    def test_publish(self):
        self.__publish('test', 'test message')

    def test_auth_publish(self):
        self.__publish('auth:test', 'test message')

    def test_private_publish(self):
        self.__publish('private:'+self.subscriber_id, 'test message')

    def __publish(self, stream, msg):
        handle = generate_handle()
        msg = {'handle' : handle, 'msg' : msg}
        json_msg = json.dumps(msg)
        self.assertTrue(self.tambur.publish(stream, json_msg))
        r = self.s.get('http://wsbot.tambur.io/results?handle='+handle)
        results = json.loads(r.text)
        self.assertEqual(len(results), 1)
        if stream.startswith('private'):
            self.assertEqual(results[0][handle], {'private' : msg})
        else:
            self.assertEqual(results[0][handle], {stream : msg})

class TamburModeTokenTest(unittest.TestCase):
    def setUp(self):
        self.tambur = Tambur(api_key='30af96de47e3c58329045ff136a4a3ea', app_id='ws-bot-1', secret='wsbot')

    def test_generate_auth_token(self):
        stream = 'test'
        subscriber_id = 'a0629978-28d8-4fd4-b862-f67e9b6dfd8f'
        token = self.tambur.generate_auth_token(stream, subscriber_id)
        self.assertEqual(token, '2f25ad1ce5afab906cc582b6254a912590c60f73')

    def test_generate_presence_token(self):
        stream = 'test'
        user = 'test_user'
        subscriber_id = 'a0629978-28d8-4fd4-b862-f67e9b6dfd8f'
        token = self.tambur.generate_presence_token(stream, user, subscriber_id)
        self.assertEqual(token, 'dcadf9659116ebbe024a4cd5ae12bde48d95408e')

    def test_generate_direct_token(self):
        stream = 'test'
        user = 'test_user'
        subscriber_id = 'a0629978-28d8-4fd4-b862-f67e9b6dfd8f'
        token = self.tambur.generate_direct_token(stream, user, subscriber_id)
        self.assertEqual(token, '2403374744295f5d22e3f999d4eb85b3f689c6b2')

def generate_handle():
    return ''.join(random.choice('ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789') for x in range(6))

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name = 'tambur',
    version = '0.1.0',
    description = 'Tambur.io Client',
    author = 'Andre Graf',
    author_email = 'graf@tambur.io',
    url = 'http://github.com/tamburio/python-tambur',
    packages = find_packages(),
    install_requires = ['requests'],
    classifiers = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
